@@ -430,6 +430,36 @@ def update_data(d):
         >>> update_data(d)
         >>> print getVar('TEST', d)
         local
+
+        CopyMonster:
+        >>> e = d.createCopy()
+        >>> setVar('TEST_foo', 'foo', e)
+        >>> update_data(e)
+        >>> print getVar('TEST', e)
+        local
+
+        >>> setVar('OVERRIDES', 'arm:ramses:local:foo', e)
+        >>> update_data(e)
+        >>> print getVar('TEST', e)
+        foo
+
+        >>> f = d.createCopy()
+        >>> setVar('TEST_moo', 'something', f)
+        >>> setVar('OVERRIDES', 'moo:arm:ramses:local:foo', e)
+        >>> update_data(e)
+        >>> print getVar('TEST', e)
+        foo
+
+
+        >>> h = init()
+        >>> setVar('SRC_URI', 'file://append.foo;patch=1 ', h)
+        >>> g = h.createCopy()
+        >>> setVar('SRC_URI_append_arm', 'file://other.foo;patch=1', g)
+        >>> setVar('OVERRIDES', 'arm:moo', g)
+        >>> update_data(g)
+        >>> print getVar('SRC_URI', g)
+        file://append.foo;patch=1 file://other.foo;patch=1
+
     """
     debug(2, "update_data()")
 
@@ -460,7 +490,7 @@ def update_data(d):
         l    = len(o)+1
 
         # see if one should even try
-        if not o in d._seen_overrides:
+        if not d._seen_overrides.has_key(o):
             continue
 
         vars = d._seen_overrides[o]
@@ -472,7 +502,7 @@ def update_data(d):
                 note ("Untracked delVar")
 
     # now on to the appends and prepends
-    if '_append' in d._special_values:
+    if d._special_values.has_key('_append'):
         appends = d._special_values['_append'] or []
         for append in appends:
             for (a, o) in getVarFlag(append, '_append', d) or []:
@@ -487,7 +517,7 @@ def update_data(d):
                 setVar(append, sval, d)
 
 
-    if '_prepend' in d._special_values:
+    if d._special_values.has_key('_prepend'):
         prepends = d._special_values['_prepend'] or []
 
         for prepend in prepends:
